#ifndef PERCEPTIONS_DISPLAY_H
#define PERCEPTIONS_DISPLAY_H

#ifndef Q_MOC_RUN
#include <vector>

#include <rviz/message_filter_display.h>
#include <multiception/Perceptions.h>
#endif

namespace Ogre
{
class SceneNode;
}

namespace rviz
{
class ColorProperty;
class FloatProperty;
class IntProperty;
}

// All the source in this plugin is in its own namespace. This is not
// required but is good practice.
namespace rviz_multiception
{

class PerceptionVisual;

// Here we declare our new subclass of rviz::Display. Every display
// which can be listed in the "Displays" panel is a subclass of
// rviz::Display.
//
// The PerceptionsDisplay class just implements the vector,
// editable parameters, and Display subclass machinery. The visuals
// themselves are represented by a separate class, PerceptionVisual. The
// idiom for the visuals is that when the objects exist, they appear
// in the scene, and when they are deleted, they disappear.
class PerceptionsDisplay: public rviz::MessageFilterDisplay<multiception::Perceptions>
{
Q_OBJECT
public:
	// Constructor. pluginlib::ClassLoader creates instances by calling
	// the default constructor, so make sure you have one.
	PerceptionsDisplay();
	virtual ~PerceptionsDisplay();

	// Overrides of protected virtual functions from Display. As much
	// as possible, when Displays are not enabled, they should not be
	// subscribed to incoming data and should not show anything in the
	// 3D view. These functions are where these connections are made
	// and broken.
protected:
	virtual void onInitialize();

	// A helper to clear this display back to the initial state.
	virtual void reset();

	// Qt slots connected to signals indicating changes in the user-editable properties.
private Q_SLOTS:
	void updateColorAndAlpha();

private:
	// Function to handle an incoming ROS message.
	void processMessage(const multiception::Perceptions::ConstPtr& msg);

	// Storage for the list of visuals and the original message to avoid NULL pointers in visuals.
	multiception::Perceptions::ConstPtr msg_;
	std::vector<boost::shared_ptr<PerceptionVisual> > visuals_;

	// User-editable property variables.
	rviz::ColorProperty* color_property_;
	rviz::FloatProperty* alpha_property_;
	rviz::IntProperty* history_length_property_;
};

} // namespace rviz_multiception

#endif // PERCEPTIONS_DISPLAY_H
